/*
 *  C++ Performance API wrapper tests.
 *
 *  Copyright 2012 Aurelian Melinte. 
 *  Released under GPL 3.0 or later. 
 */


#include "papi.hpp"


const int nlines = 196608;
const int ncols  = 64;
char ctrash[nlines][ncols];


//------------------------------------------------------------------------------
#define THRESHOLD 10000

void computation_mult()
{
   double tmp=1.0;
   int i=1;
   for( i = 1; i < THRESHOLD; i++ )
   {
      tmp = tmp*i;
   }
}

void computation_add()
{
   int tmp = 0;
   int i=0;

   for( i = 0; i < THRESHOLD; i++ )
   {
      tmp = tmp + i;
   }

}

//------------------------------------------------------------------------------


int main()
{
   std::cout << "*\n"
                "* Baseline \n"
                "*\n";
   {
       papi::counters<papi::stdout_print> pc("Baseline add()");
       computation_add();
   }
   {
       papi::counters<papi::stdout_print> pc("Baseline mult()");
       computation_mult();
   }
       
   std::cout << "*\n"
                "* add & mult \n"
                "*\n";
   {
       papi::counters<papi::stdout_print> pc("add+mult no intermediate print");
       
       {
           papi::counters<papi::nop> pc2("add & no print");
           computation_add();
       }
       
       {
           papi::counters<papi::nop> pc2("mult & no print");
           computation_mult();
       }
   }
   {
       papi::counters<papi::stdout_print> pc("add+mult & print");
       
       {
           papi::counters<papi::stdout_print> pc2("add & print");
           computation_add();
       }
       
       {
           papi::counters<papi::stdout_print> pc2("mult & print");
           computation_mult();
       }
   }
    
   std::cout << "*\n"
                "* Atomics \n"
                "*\n";
   {
       int i = 0;
       papi::counters<papi::stdout_print> pc("Plain int");
       for (int i = 0; i < THRESHOLD; ++i) {
           i = i++;
       }
   }
   {
       std::atomic<int> i(0);
       papi::counters<papi::stdout_print> pc("Atomic int");
       for (int i = 0; i < THRESHOLD; ++i) {
           i++;
       }
   }
   
   std::cout << "*\n"
                "* Data cache trashing \n"
                "*\n";
   {
       int x;
       papi::counters<papi::stdout_print> pc("By line");
       for (int l = 0; l < nlines; ++l) {
           for (int c = 0; c < ncols; ++c) {
               x = ctrash[l][c];
           }
       }
   }
   {
       int x;
       papi::counters<papi::stdout_print> pc("by column");
       for (int c = 0; c < ncols; ++c) {
           for (int l = 0; l < nlines; ++l) {
               x = ctrash[l][c];
           }
       }
   }
   
   std::cout << "*\n"
                "* Data cache trashing - placate optimizer \n"
                "*\n";
   {
       int x;
       papi::counters<papi::stdout_print> pc("by column");
       for (int c = 0; c < ncols; ++c) {
           for (int l = 0; l < nlines; ++l) {
               x = ctrash[l][c];
               ctrash[l][c] = x + 1;
           }
       }
   }
   {
       int x;
       papi::counters<papi::stdout_print> pc("By line");
       for (int l = 0; l < nlines; ++l) {
           for (int c = 0; c < ncols; ++c) {
               x = ctrash[l][c];
               ctrash[l][c] = x + 1;
           }
       }
   }
   
   exit(0);    
}

